﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using OptimaJet.Workflow.Core.Runtime;
using OptimaJet.Workflow.Core.Subprocess;
using WorkflowApp;

namespace WorkflowAppConsole
{
    public class Program
    {
        private static string _currentUser = string.Empty;
        private const string SchemeCode = "SimpleWF";
        static Guid? _processId = null;
        static void Main(string[] args)
        {
            Console.WriteLine("Operation:");
            Console.WriteLine("0 - Set current user");
            Console.WriteLine("1 - CreateInstance");
            Console.WriteLine("2 - GetAvailableCommands");
            Console.WriteLine("3 - ExecuteCommand");
            Console.WriteLine("4 - GetAvailableState");
            Console.WriteLine("5 - SetState");
            Console.WriteLine("6 - DeleteProcess");
            Console.WriteLine("9 - Exit");

            Console.WriteLine("The process isn't created.");
            CreateInstance();
            
            do
            {
                if (_processId.HasValue)
                {
                    Console.WriteLine("ProcessId = '{0}'. CurrentState: {1}, CurrentActivity: {2}",
                        _processId,
                        WorkflowInit.Runtime.GetCurrentStateName(_processId.Value),
                        WorkflowInit.Runtime.GetCurrentActivityName(_processId.Value));

                    var processTree = WorkflowInit.Runtime.GetProcessInstancesTree(_processId.Value);
                    if (processTree != null)
                    {
                        var current = processTree.Root;
                        var level = "->";
                        WriteSubprocesses(current, level);
                    }
                }

                

                if (!string.IsNullOrEmpty(_currentUser))
                    Console.WriteLine("Current user = {0}.", _currentUser);
                else
                {
                    Console.WriteLine("Current user is undefined.");
                }

                Console.Write("Enter code of operation:");

                char operation = Console.ReadLine().FirstOrDefault();
                
                switch(operation)
                {
                    case '0':
                        SetUser();
                        break;
                    case '1':
                        CreateInstance();
                        break;
                    case '2':
                        GetAvailableCommands();
                        break;
                    case '3':
                        ExecuteCommand();
                        break;
                    case '4':
                        GetAvailableState();
                        break;
                    case '5':
                        SetState();
                        break;
                    case '6':
                        DeleteProcess();
                        break;
                    case '9':
                        return;
                    default:
                        Console.WriteLine("Unknown code. Please, repeat.");
                        break;
                }

                Console.WriteLine();
            } while (true);
        }

        private static void WriteSubprocesses(ProcessInstancesTree current, string level)
        {
            foreach (var child in current.Children)
            {
                Console.WriteLine("{0}SubProcessId = '{1}'. CurrentState: {2}, CurrentActivity: {3}",
                    level,
                    child.Id,
                    WorkflowInit.Runtime.GetCurrentStateName(child.Id),
                    WorkflowInit.Runtime.GetCurrentActivityName(child.Id));

                WriteSubprocesses(child,level + "->");
            }
        }

        private static void SetUser()
        {
            Console.Write("Enter user's id:");
            var readLine = Console.ReadLine();
            if (readLine != null)
            {
                _currentUser = readLine.Trim();
            }
        }

        private static void CreateInstance()
        {
            _processId = Guid.NewGuid();

            try
            {
                WorkflowInit.Runtime.CreateInstance(SchemeCode, _processId.Value);
                Console.WriteLine("CreateInstance - OK.", _processId);
            }
            catch (Exception ex)
            {
                Console.WriteLine("CreateInstance - Exception: {0}, {1}", ex.Message, ex.InnerException != null ? ex.InnerException.Message : string.Empty);
                _processId = null;
            }
        }

        private static void GetAvailableCommands()
        {
            if(_processId == null)
            {
                Console.WriteLine("The process isn't created. Please, create process instance.");
                return;
            }

            var commands = WorkflowInit.Runtime.GetAvailableCommands(_processId.Value, _currentUser);
            
            Console.WriteLine("Available commands:");
            if (commands.Count() == 0)
            {
                Console.WriteLine("Not found!");
            }
            else
            {
                foreach (var command in commands)
                {
                    Console.WriteLine("- {0} (LocalizedName:{1}, Classifier:{2})", command.CommandName, command.LocalizedName, command.Classifier);
                }
            }
        }

        private static void ExecuteCommand()
        {
            if (_processId == null)
            {
                Console.WriteLine("The process isn't created. Please, create process instance.");
                return;
            }

            WorkflowCommand command = null;

            do
            {
                GetAvailableCommands();
                Console.Write("Enter command:");
                var readLine = Console.ReadLine();
                if (readLine != null)
                {
                    var commandName = readLine.ToLower().Trim();
                    if (commandName == string.Empty)
                        return;

                    command = WorkflowInit.Runtime.GetAvailableCommands(_processId.Value, _currentUser).FirstOrDefault(c => c.CommandName.Trim().ToLower() == commandName);
                }
                if (command == null)
                    Console.WriteLine("The command isn't found.");
            } while (command == null);

            WorkflowInit.Runtime.ExecuteCommand(command,_currentUser,_currentUser);
            Console.WriteLine("ExecuteCommand - OK.");
        }

        private static void GetAvailableState()
        {
            if (_processId == null)
            {
                Console.WriteLine("The process isn't created. Please, create process instance.");
                return;
            }

            var states = WorkflowInit.Runtime.GetAvailableStateToSet(_processId.Value, Thread.CurrentThread.CurrentCulture);

            Console.WriteLine("Available state to set:");

            var workflowStates = states as WorkflowState[] ?? states.ToArray();

            if (!workflowStates.Any())
            {
                Console.WriteLine("Not found!");
            }
            else
            {
                foreach (var state in workflowStates)
                {
                    Console.WriteLine("- {0}", state.Name);
                }
            }
        }

        private static void SetState()
        {
            if (_processId == null)
            {
                Console.WriteLine("The process isn't created. Please, create process instance.");
                return;
            }

            string stateName = string.Empty;
            WorkflowState state; 
            do
            {
                GetAvailableState();
                Console.Write("Enter state:");
                var readLine = Console.ReadLine();
                if (readLine != null) stateName = readLine.ToLower().Trim();
                if (stateName == string.Empty)
                    return;

                state = WorkflowInit.Runtime.GetAvailableStateToSet(_processId.Value, Thread.CurrentThread.CurrentCulture).FirstOrDefault(c => c.Name.Trim().ToLower() == stateName);

                if (state == null)
                    Console.WriteLine("The state isn't found.");
                else
                    break;
            } while (true);

            WorkflowInit.Runtime.SetState(_processId.Value, _currentUser, _currentUser, state.Name, new Dictionary<string, object>());
            Console.WriteLine("SetState - OK.");
        }

        private static void DeleteProcess()
        {
            if (_processId == null)
            {
                Console.WriteLine("The process isn't created. Please, create process instance.");
                return;
            }

            WorkflowInit.Runtime.DeleteInstance(_processId.Value);
            Console.WriteLine("DeleteProcess - OK.");
            _processId = null;
        }
    }
}
